/******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation
 * All rights reserved.
 * This program and the accompanying materials
 * are made available under the terms of the BSD License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Contributors:
 *     IBM Corporation - initial implementation
 *****************************************************************************/

#ifndef _TYPES_H_
#define _TYPES_H_

typedef unsigned char 		u08_t;
typedef unsigned short 		u16_t;
typedef unsigned int 		u32_t;
typedef unsigned long long 	u64_t;

typedef signed char 		i08_t;
typedef signed short 		i16_t;
typedef signed int 		i32_t;
typedef signed long long 	i64_t;

#endif
